@@ -9,7 +9,7 @@
 #
 package Config::Model;
 {
-  $Config::Model::VERSION = '2.011';
+  $Config::Model::VERSION = '2.021';
 }
 use Any::Moose ;
 use namespace::autoclean;
@@ -96,7 +96,7 @@ Config::Model - Create tools to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.011
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -1994,10 +1994,16 @@ sub get_model_doc {
             push @elt, $self->get_element_description($elt_info) , '' ;
 
             foreach ($elt_info,$elt_info->{cargo}) { 
-                my $ccn = $_->{config_class_name};
-                next unless defined $ccn ;
-                push @classes, $ccn ;
-                $see_also{$ccn} = 1;
+                if (my $ccn = $_->{config_class_name}) {
+                    push @classes, $ccn ;
+                    $see_also{$ccn} = 1;
+                }
+                if (my $migr = $_->{migrate_from}) {
+                    push @elt, $self->get_migrate_doc ($elt_name,'is migrated with',$migr);
+                }
+                if (my $migr = $_->{migrate_values_from}) {
+                    push @elt, "Note: $elt_name values are migrated from '$migr'",'';
+                }
             }
         }
 
@@ -2015,7 +2021,7 @@ sub get_model_doc {
         }
 
         my @see_also =  (
-            "=head1 SEE ALSO",'',"=over",'',"=item *",'',"L<config-edit>",'',
+            "=head1 SEE ALSO",'',"=over",'',"=item *",'',"L<cme>",'',
             ( map { ( "=item *",'',"L<Config::Model::models::$_>",'') ; } sort keys %see_also ),
             "=back",'') ;
 
@@ -2024,6 +2030,27 @@ sub get_model_doc {
     return \%result ;
 }
 
+#
+# New subroutine "get_migrate_doc" extracted - Tue Jun  5 13:31:20 2012.
+#
+sub get_migrate_doc {
+    my ( $self, $elt_name, $desc, $migr ) = @_;
+
+    my $mv    = $migr->{variables};
+    my $mform = $migr->{formula};
+
+    if ( $migr->{use_eval} ) { $mform =~ s/^/ /mg; $mform = "\n\n$mform\n\n"; }
+    else                     { $mform = "'C<$mform>' " }
+
+    my $mdoc = "Note: $elt_name $desc ${mform}and with "
+      . join( ", ", map { qq!\$$_ => "C<$mv->{$_}>"! } keys %$mv );
+    if (my $rep = $migr->{replace}) {
+        $mdoc .= ' and '.join( ", ", map { qq!'C<\$replace{$_}>' => "C<$rep->{$_}>"! } keys %$rep );
+    }
+    
+    return ( $mdoc, '' );
+}
+
 sub get_element_description {
     my ( $self, $elt_info ) = @_;
 
@@ -2040,6 +2067,11 @@ sub get_element_description {
     if ($desc) {
         $desc .= '. ' if $desc =~ /\w$/ ;
     }
+
+    if (my $status = $elt_info->{status}) {
+        $desc .= 'B<'.ucfirst($status).'> ';
+    }  
+
     
     my $info = $elt_info->{mandatory} ? 'Mandatory. ' : 'Optional. ' ;
 
@@ -2065,7 +2097,7 @@ sub get_element_value_help {
     
     my $help_text = "\n\nHere are some explanations on the possible values:\n\n=over\n\n" ;
     foreach my $v (sort keys %$help) {
-        $help_text .= "=item $v\n\n$help->{$v}\n\n" ;
+        $help_text .= "=item '$v'\n\n$help->{$v}\n\n" ;
     }
     
     return $help_text."=back\n\n" ;
