@@ -9,17 +9,22 @@
 #
 package Config::Model::AnyThing;
 {
-  $Config::Model::AnyThing::VERSION = '2.005';
+  $Config::Model::AnyThing::VERSION = '2.021';
 }
 
 use Any::Moose ;
 use namespace::autoclean;
 
+# FIXME: must cleanup warp mechanism to implement this
+# use Any::Moose 'X::StrictConstructor' ;
+
 use Pod::POM ;
 use Carp;
 use Log::Log4perl qw(get_logger :levels);
+use 5.10.1;
 
 my $logger = get_logger("Anything") ;
+my $change_logger = get_logger("Anything::Change") ;
 
 has element_name => ( is => 'ro', isa => 'Str') ;
 has parent => (is => 'ro', isa => 'Config::Model::Node' , weak_ref => 1);
@@ -61,13 +66,21 @@ has location => (is => 'ro', isa => 'Str' , builder => '_location', lazy => 1);
 
 sub notify_change {	
     my $self = shift ;
+    my %args =  @_ ;
 
-    $self->container->notify_change(
-	needs_write => 1 , # may be overridden by caller
-	@_, 
-	name => $self->element_name, 
-	index => $self->index_value
-    );
+    return if $self->instance->initial_load  and not $args{really};
+
+    $change_logger->debug("called for ",$self->name, " from ", join(' ',caller) ) 
+        if $change_logger->is_debug ;
+
+    # needs_save may be overridden by caller
+    $args{needs_save} //= 1 ;
+    $args{path} //= $self->location ;
+    $args{name} //= $self->element_name if $self->element_name ;
+    $args{index} //= $self->index_value if $self->index_value ;
+    
+    # beter use %args instead of @_ to forward arguments. %args eliminates duplicated keys
+    $self->container->notify_change(%args);
 }
 
 
@@ -561,7 +574,7 @@ Config::Model::AnyThing - Base class for configuration tree item
 
 =head1 VERSION
 
-version 2.005
+version 2.021
 
 =head1 SYNOPSIS
 
