@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2010 Dominique Dumont.
+#    Copyright (c) 2010-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -26,12 +26,12 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Backend::ShellVar ;
-BEGIN {
-  $Config::Model::Backend::ShellVar::VERSION = '1.226';
+{
+  $Config::Model::Backend::ShellVar::VERSION = '2.021';
 }
 
 use Carp;
-use Moose;
+use Any::Moose ;
 use Config::Model::Exception ;
 use UNIVERSAL ;
 use File::Path;
@@ -65,21 +65,13 @@ sub read {
     # try to get global comments (comments before a blank line)
     $self->read_global_comments(\@lines,'#') ;
 
-    my @comments ;
-    foreach (@lines) {
-        next if /^##/ ;		  # remove comments added by Config::Model
-        chomp ;
-
-        my ($data,$comment) = split /\s*#\s?/ ;
-
-        push @comments, $comment        if defined $comment ;
-
-        if (defined $data and $data ) {
-            $data .= '#"'.join("\n",@comments).'"' if @comments ;
-            $logger->debug("Loading:$data\n");
-            $self->node->load(step => $data, check => $check) ;
-            @comments = () ;
-        }
+    my @assoc = $self->associates_comments_with_data( \@lines, '#' ) ;
+    foreach my $item (@assoc) {
+        my ($data,$c) = @$item;
+        my $load = qq!$data! ;
+        $load .= qq!#"$c"! if $c ;
+        $logger->debug("Loading:$load\n");
+        $self->node->load(step => $load, check => $check) ;
     }
 
     return 1 ;
@@ -103,45 +95,27 @@ sub write {
 
     croak "Undefined file handle to write" unless defined $ioh;
 
-    $ioh->print("## This file was written by Config::Model\n");
-    $ioh->print("## You may modify the content of this file. Configuration \n");
-    $ioh->print("## modifications will be preserved. Modifications in\n");
-    $ioh->print("## comments may be mangled.\n##\n");
-
-    # write global comment
-    my $global_note = $node->annotation ;
-    if ($global_note) {
-        map { $ioh->print("# $_\n") } split /\n/,$global_note ;
-        $ioh->print("\n") ;
-    }
 
+    my @to_write ;
     # Using Config::Model::ObjTreeScanner would be overkill
     foreach my $elt ($node->get_element_name) {
         my $obj =  $node->fetch_element($elt) ;
         my $v = $node->grab_value($elt) ;
 
-        # write some documentation in comments
-        my $help = $node->get_help(summary => $elt);
-        my $upstream_default = $obj -> fetch('upstream_default') ;
-        $help .=" ($upstream_default)" if defined $upstream_default;
-        $ioh->print("## $elt: $help\n") if $help;
+        next unless defined $v ;
 
-
-        # write annotation
-        my $note = $obj->annotation ;
-        if ($note) {
-            map { $ioh->print("# $_\n") } split /\n/,$note ;
-        }
-
-        # write value
-        $ioh->print(qq!$elt="$v"\n!) if defined $v ;
-        $ioh->print("\n") if defined $v or $help;
+        push @to_write, [ qq!$elt="$v"!, $obj->annotation ] ;
     }
 
+    if (@to_write) {
+        $self->write_global_comment($ioh,'#') ;
+        map { $self->write_data_and_comments($ioh,'#',@$_) ; } @to_write ;
+    }
+    
     return 1;
 }
 
-no Moose ;
+no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -150,33 +124,57 @@ __END__
 
 =head1 NAME
 
-Config::Model::Backend::ShellVar - Read and write config as a SHELLVAR data structure
+Config::Model::Backend::ShellVar - Read and write config as a C<SHELLVAR> data structure
 
 =head1 VERSION
 
-version 1.226
+version 2.021
 
 =head1 SYNOPSIS
 
-  # model declaration
-  name => 'FooConfig',
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
-  read_config  => [
-                    { backend => 'shellvar' , 
-                      config_dir => '/etc/foo',
-                      file  => 'foo.conf',      # optional
-                      auto_create => 1,         # optional
-                    }
-                  ],
+ my $model = Config::Model->new;
+ $model->create_config_class (
+    name    => "MyClass",
+    element => [ 
+        [qw/foo bar/] => {qw/type leaf value_type string/}
+    ],
 
-   element => ...
-  ) ;
+   read_config  => [
+        { 
+            backend => 'ShellVar',
+            config_dir => '/tmp',
+            file  => 'foo.conf',
+            auto_create => 1,
+        }
+    ],
+ );
 
+ my $inst = $model->instance(root_class_name => 'MyClass' );
+ my $root = $inst->config_root ;
+
+ $root->load('foo=FOO1 bar=BAR1' );
+
+ $inst->write_back ;
+
+File C<foo.conf> now contains:
+
+ ## This file was written by Config::Model
+ ## You may modify the content of this file. Configuration 
+ ## modifications will be preserved. Modifications in
+ ## comments may be mangled.
+ ##
+ foo="FOO1"
+
+ bar="BAR1"
 
 =head1 DESCRIPTION
 
 This module is used directly by L<Config::Model> to read or write the
-content of a configuration tree written with SHELLVAR syntax in
+content of a configuration tree written with C<SHELLVAR> syntax in
 C<Config::Model> configuration tree.
 
 Note that undefined values are skipped for list element. I.e. if a
@@ -204,7 +202,7 @@ When a file is read,  C<read()> will return 1.
 =head2 write ( io_handle => ... )
 
 Of all parameters passed to this write call-back, only C<io_handle> is
-used. This parameter must be L<IO::File> object alwritey opened for
+used. This parameter must be L<IO::File> object already opened for
 write. 
 
 C<write()> will return 1.
@@ -219,4 +217,4 @@ L<Config::Model>,
 L<Config::Model::AutoRead>, 
 L<Config::Model::Backend::Any>, 
 
-=cut
\ No newline at end of file
+=cut
