@@ -27,7 +27,7 @@
 
 package Config::Model::Loader;
 {
-  $Config::Model::Loader::VERSION = '2.011';
+  $Config::Model::Loader::VERSION = '2.021';
 }
 use Carp;
 use strict;
@@ -44,7 +44,7 @@ Config::Model::Loader - Load serialized data into config tree
 
 =head1 VERSION
 
-version 2.011
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -593,7 +593,7 @@ sub _load_list {
     my ($self,$node, $check,$experience,$inst,$cmdref) = @_ ;
     my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
-    my $element = $node -> fetch_element($element_name) ;
+    my $element = $node -> fetch_element(name => $element_name, check => $check) ;
 
     my $elt_type   = $node -> element_type( $element_name ) ;
     my $cargo_type = $element->cargo_type ;
@@ -667,7 +667,7 @@ sub _load_hash {
     my ($self,$node,$check,$experience,$inst,$cmdref) = @_ ;
     my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
-    my $element = $node -> fetch_element($element_name) ;
+    my $element = $node -> fetch_element(name => $element_name, check => $check ) ;
     my $cargo_type = $element->cargo_type ;
 
     if (defined $note and not defined $action) {
@@ -685,7 +685,7 @@ sub _load_hash {
     }
     
     if ($action eq '=~') {
-	my @keys = $element->get_all_indexes;
+	my @keys = $element->fetch_all_indexes;
 	my $ret ;
 	$logger->debug("_load_hash: looping with regex $id on keys @keys");
 	$id =~ s!^/!!;
@@ -765,7 +765,7 @@ sub _load_leaf {
     my ($self,$node,$check,$experience,$inst,$cmdref) = @_ ;
     my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
-    my $element = $node -> fetch_element($element_name) ;
+    my $element = $node -> fetch_element(name => $element_name, check => $check) ;
     $self->_load_note($element, $note, $inst, $cmdref);
 
     if (defined $action and $action eq '~' and $element->isa('Config::Model::Value')) {
