@@ -9,7 +9,7 @@
 #
 package Config::Model::Node;
 {
-  $Config::Model::Node::VERSION = '2.011';
+  $Config::Model::Node::VERSION = '2.021';
 }
 
 use Any::Moose ;
@@ -27,6 +27,7 @@ use Config::Model::Describe;
 use Config::Model::BackendMgr;
 use Log::Log4perl qw(get_logger :levels);
 use Storable qw/dclone/ ;
+use List::MoreUtils qw(insert_after_string);
 
 extends qw/Config::Model::AnyThing/;
 
@@ -106,6 +107,8 @@ has auto_read => ( is => 'rw' , isa => 'HashRef' ) ;
 has model => ( is => 'rw' , isa => 'HashRef' ) ;
 has needs_save => ( is => 'rw', isa => 'Bool', default => 0 ) ;
 
+has backend_mgr => ( is => 'ro', isa => 'Maybe[Config::Model::BackendMgr]') ;
+
 sub BUILD {
     my $self = shift;
 
@@ -342,10 +345,11 @@ sub init {
     return
       unless defined $model->{read_config}
           or defined $model->{write_config};
-          
-    $self->instance->initial_load_start ;
     
-    $self->{bmgr} ||= Config::Model::BackendMgr->new( node => $self );
+    my $initial_load_backup = $self->instance->initial_load ;      
+    $self->instance->initial_load_start;
+    
+    $self->{backend_mgr} ||= Config::Model::BackendMgr->new( node => $self );
 
     my $ar = $self->{auto_read};
 
@@ -362,13 +366,13 @@ sub init {
     if ( $model->{write_config} ) {
 
         # setup auto_write, write_config_dir is obsolete
-        $self->{bmgr}->auto_write_init(
+        $self->backend_mgr->auto_write_init(
             write_config     => $model->{write_config},
             write_config_dir => $model->{write_config_dir},
         );
     }
 
-    $self->instance->initial_load_stop ;
+    $self->instance->initial_load( $initial_load_backup );
 }
 
 sub read_config_data {
@@ -383,7 +387,7 @@ sub read_config_data {
 
     # setup auto_read, read_config_dir is obsolete
     # may use an overridden config file
-    $self->{bmgr}->read_config_data(
+    $self->backend_mgr->read_config_data(
         read_config     => $model->{read_config},
         check           => $args{check},
         read_config_dir => $model->{read_config_dir},
@@ -394,14 +398,15 @@ sub read_config_data {
 
 sub notify_change {	
     my $self = shift ;
+    my %args = @_ ;
 
-    return if $self->instance->initial_load ;
+    return if $self->instance->initial_load  and not $args{really};
 
-    # $logger->debug("called while needs_write is ",$self->needs_save,
-	# " for ",$self->name) 
-	# if $logger->is_debug ;
+    $logger->debug("called while needs_write is ",$self->needs_save,
+	" for ",$self->name) 
+	if $logger->is_debug ;
 
-    if (defined $self->{bmgr}) {
+    if (defined $self->backend_mgr) {
         $self->needs_save(1) ; # will trigger a save in config_file
 	$self->SUPER::notify_change( @_, needs_save => 0 ) ;
     }
@@ -422,13 +427,13 @@ sub write_back {
             $self->location,")\n";
     }
     
-    $self->{bmgr}->write_back(%args) if $self->needs_save or $force_write;
+    $self->backend_mgr->write_back(%args) if $self->needs_save or $force_write;
 }
 
 sub is_auto_write_for_type {
     my $self = shift ;
-    return 0 unless defined $self->{bmgr} ;
-    return $self->{bmgr}->is_auto_write_for_type(@_) ;
+    return 0 unless defined $self->backend_mgr ;
+    return $self->backend_mgr->is_auto_write_for_type(@_) ;
 }
 
 
@@ -898,6 +903,7 @@ sub reset_accepted_element_model {
 
     my $model = dclone $accept_model ;
     delete $model->{name_match} ;
+    my $accept_after = delete $model->{accept_after} ;
     
     foreach my $info_to_move (qw/description summary/) {
         my $moved_data = delete $model->{$info_to_move}  ;
@@ -913,8 +919,13 @@ sub reset_accepted_element_model {
     $self->{model}{element}{$element_name} = $model ;
 
     #add to element list...
-    push @{$self->{model}{element_list}}, $element_name;
-
+    if ($accept_after) {
+        insert_after_string($accept_after, $element_name, @{$self->{model}{element_list}} );
+    }
+    else {
+        push @{$self->{model}{element_list}}, $element_name;
+    }
+    
     return ($model);
 }
 
@@ -1166,7 +1177,7 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.011
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -1368,6 +1379,25 @@ Example:
 
 All C<element> parameters can be used in specifying accepted parameters.
 
+The parameter C<accept_after> to specify where to insert the accepted element.
+This will not change much the behavior of the tree, but it will help generate
+user interface easier to use.
+
+Example:
+
+ element => [
+    'Bug' => { type => 'leaf', value_type => 'uniline' } ,
+ ]
+ accept => [
+    'Bug-.*' =>  {
+         value_type => 'uniline',
+         type => 'leaf'
+         accept_after => 'Bug' ,
+    }
+ ]
+
+The model snippet above will ensure that C<Bug-Debian> will be shown right after C<bug>.
+
 =for html
 <p>For more information, see <a href="http://ddumont.wordpress.com/2010/05/19/improve-config-upgrade-ep-02-minimal-model-for-opensshs-sshd_config/">this blog<\a>.</p>
 
@@ -1737,6 +1767,10 @@ Load configuration data with a hash ref (first parameter). The hash ref key must
 the available elements of the node. The hash ref structure must match
 the structure of the configuration model.
 
+=head2 needs_save
+
+return 1 if one of the elements of the node's sub-tree has been modified.
+
 =head1 Serialization
 
 =head2 dump_tree ( ... )
