@@ -1,13 +1,13 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
-#    Copyright (c) 2006-2010 Dominique Dumont.
+#
+#    Copyright (c) 2006-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -26,6 +26,9 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::ObjTreeScanner ;
+{
+  $Config::Model::ObjTreeScanner::VERSION = '2.021';
+}
 use strict ;
 use Config::Model::Exception ;
 use Scalar::Util qw/blessed/ ;
@@ -33,7 +36,6 @@ use Carp::Assert::More ;
 use Carp;
 use warnings ;
 
-our $VERSION="1.202";
 
 use Carp qw/croak confess cluck/;
 
@@ -43,73 +45,74 @@ Config::Model::ObjTreeScanner - Scan config tree and perform call-backs
 
 =head1 VERSION
 
-version 1.205
+version 2.021
 
 =head1 SYNOPSIS
 
- use Config::Model::ObjTreeScanner ;
+ use Config::Model ;
+ use Log::Log4perl qw(:easy) ;
+ Log::Log4perl->easy_init($WARN);
 
  # define configuration tree object
- my $root = ... ;
+ my $model = Config::Model->new ;
+ $model ->create_config_class (
+    name => "MyClass",
+    element => [ 
+        [qw/foo bar/] => { 
+            type => 'leaf',
+            value_type => 'string'
+        },
+        baz => { 
+            type => 'hash',
+            index_type => 'string' ,
+            cargo => {
+                type => 'leaf',
+                value_type => 'string',
+            },
+        },
+        
+    ],
+ ) ;
+
+ my $inst = $model->instance(root_class_name => 'MyClass' );
+
+ my $root = $inst->config_root ;
+
+ # put some data in config tree the hard way
+ $root->fetch_element('foo')->store('yada') ;
+ $root->fetch_element('bar')->store('bla bla') ;
+ $root->fetch_element('baz')->fetch_with_id('en')->store('hello') ;
+
+ # put more data the easy way
+ my $step = 'baz:fr=bonjour baz:hr="dobar dan"';
+ $root->load( step => $step ) ;
 
  # define leaf call back
- my disp_leaf = sub { 
+ my $disp_leaf = sub { 
       my ($scanner, $data_ref, $node,$element_name,$index, $leaf_object) = @_ ;
-      $$data_ref .= "$element_name = ", $leaf_object->fetch ;
+      $$data_ref .= "disp_leaf called for '". $leaf_object->name. 
+	"' value '".$leaf_object->fetch."'\n";
     } ;
 
  # simple scanner, (print all values with 'beginner' experience
- $scan = Config::Model::ObjTreeScanner-> new
-  (
-   leaf_cb               => \&disp_leaf, # only mandatory parameter
-  ) ;
+ my $scan = Config::Model::ObjTreeScanner-> new (
+   leaf_cb => $disp_leaf, # only mandatory parameter
+ ) ;
 
  my $result = '';
-
- $scan->scan_node(\$result, $root) ;
- 
-
- # For a more complex scanner
-
- $scan = Config::Model::ObjTreeScanner-> new
-  (
-   fallback => 'none',     # all callback must be defined
-   experience => 'master', # consider all values
-
-   # node callback
-   node_content_cb               => \&disp_obj_elt ,
-
-   # element callback
-   list_element_cb       => \&disp_hash    ,
-   check_list_element_cb => \&disp_hash    ,
-   hash_element_cb       => \&disp_hash    ,
-   node_element_cb       => \&disp_obj     ,
-
-   # leaf callback
-   leaf_cb               => \&disp_leaf,
-   enum_value_cb         => \&disp_leaf,
-   integer_value_cb      => \&disp_leaf,
-   number_value_cb       => \&disp_leaf,
-   boolean_value_cb      => \&disp_leaf,
-   string_value_cb       => \&disp_leaf,
-   uniline_value_cb      => \&disp_leaf,
-   reference_value_cb    => \&disp_leaf,
-
-   # call-back when going up the tree
-   up_cb                 => sub {} ,
-  ) ;
-
  $scan->scan_node(\$result, $root) ;
+ print $result ;
 
 =head1 DESCRIPTION
 
 This module creates an object that will explore (depth first) a
 configuration tree.
 
-For each part of the configuration tree, ObjTreeScanner object will
-call-back one of the subroutine reference passed during construction.
+For each part of the configuration tree, ObjTreeScanner object will call
+one of the subroutine reference passed during construction. (a call-back
+or a hook)
 
-Call-back routines will be called:
+Call-back and hook routines will be called:
 
 =over
 
@@ -131,9 +134,12 @@ simple leaf or another node.
 
 To continue the exploration, these call-backs must also call the
 scanner. (i.e. perform another call-back). In other words the user's
-subroutine and the scanner plays a game of ping-pong until the tree is
+subroutine and the scanner play a game of ping-pong until the tree is
 completely explored.
 
+Hooks routines are not required to resume the exploration, i.e. to call
+the scanner. This will be done after the hook routine has returned.
+
 The scanner provides a set of default callback for the nodes. This
 way, the user only have to provide call-backs for the leaves.
 
@@ -146,8 +152,8 @@ ref may be used to store whatever result you want.
 =head2 new ( ... )
 
 One way or another, the ObjTreeScanner object must be able to find all
-callback for all the items of the tree. All the possible call-back are
-listed below:
+callback for all the items of the tree. All the possible call-back and
+hooks are listed below:
 
 =over
 
@@ -160,7 +166,11 @@ C<reference_value_cb>
 
 =item node callback:
 
-C<node_content_cb> 
+C<node_content_cb> , C<node_dispatch_cb>
+
+=item node hooks:
+
+C<node_content_hook>
 
 =item element callback:
 
@@ -168,15 +178,18 @@ All these call-backs are called on the elements of a node:
 C<list_element_cb>, C<check_list_element_cb>, C<hash_element_cb>,
 C<node_element_cb>, C<node_content_cb>.
 
+=item element hooks:
+
+C<list_element_hook>, C<hash_element_hook>.
+
 =back
 
-The user may specify all of them by passing the sub ref to the
+The user may specify all of them by passing a sub ref to the
 constructor:
 
    $scan = Config::Model::ObjTreeScanner-> new
   (
-   # node callback
-   list_element_cb => sub ,
+   list_element_cb => sub { ... },
    ...
   )
 
@@ -206,6 +219,10 @@ Set the privilege level used for the scan (default 'beginner').
 
 Whether to create the configuration items while scan (default is 1).
 
+=item check 
+
+C<yes>, C<no> or C<skip>.
+
 =back
 
 =head1 Callback prototypes
@@ -273,6 +290,11 @@ Example:
      # note: scan_list and scan_hash are equivalent
   }
 
+=head2 List element hook
+
+C<hash_element_hook>: Works like the list element callback. Except that the calls to 
+C<scan_list> are not required. This will be done once the hook returns.
+
 =head2 Check list element callback
 
 C<check_list_element_cb>: Like C<list_element_cb>, but called on a
@@ -302,6 +324,11 @@ Example:
      map {$scanner->scan_hash($data_ref,$node,$element_name,$_)} @keys ;
   }
 
+=head2 Hash element hook
+
+C<hash_element_hook>: Works like the hash element callback. Except that the calls to 
+C<scan_hash> are not required. This will be done once the hook returns.
+
 =head2 Node content callback
 
 C<node_content_cb>: This call-back is called foreach node (including
@@ -313,7 +340,46 @@ C<@element_list> contains all the element names of the node.
 
 Example:
 
-  sub my_node_element_cb = { 
+  sub my_content_cb = { 
+     my ($scanner, $data_ref,$node,@element) = @_ ;
+
+     # custom code using $data_ref
+
+     # resume exploration
+     map {$scanner->scan_element($data_ref, $node,$_)} @element ;
+  }
+
+=head2 Node content hook
+
+C<node_content_hook>: This hook is called foreach node (including
+root node). Works like the node content call-back. Except that the calls to 
+C<scan_element> are not required. This will be done once the hook returns.
+
+=head2 Dispatch node callback
+
+C<node_dispatch_cb>: Any callback specified in the hash will be called for 
+each instance of the specified configuration class.
+(this may include the  root node).
+
+For instance, if you have:
+
+  node_dispach_cb => {
+    ClassA => \&my_class_a_dispatch_cb,
+    ClassB => \&my_class_b_dispatch_cb,
+  }
+
+C<&my_class_a_dispatch_cb> will be called for each instance of C<ClassA> and 
+C<&my_class_b_dispatch_cb> will be called for each instance of C<ClassB>.
+
+They will be called with the following parameters:
+
+ ($scanner, $data_ref,$node,@element_list)
+
+C<@element_list> contains all the element names of the node.
+
+Example:
+
+  sub my_class_a_dispatch_cb = { 
      my ($scanner, $data_ref,$node,@element) = @_ ;
 
      # custom code using $data_ref
@@ -338,7 +404,7 @@ C<auto_vivify> is 0.
 
 Example:
 
-  sub my_node_content_cb {
+  sub my_node_element_cb {
     my ($scanner, $data_ref,$node,$element_name,$key, $contained_node) = @_;
 
     # your custom code using $data_ref
@@ -353,7 +419,7 @@ sub new {
     my $type = shift ;
     my %args = @_;
 
-    my $self = { experience => 'beginner' , auto_vivify => 1 } ;
+    my $self = { experience => 'beginner' , auto_vivify => 1, check => 'yes' } ;
     bless $self,$type ;
 
     $self->{leaf_cb} = delete $args{leaf_cb} or
@@ -371,8 +437,9 @@ sub new {
     my @value_cb = map {$_.'_value_cb'} 
 	qw/boolean enum string uniline integer number reference/; 
 
-    foreach my $param (qw/node_element_cb hash_element_cb 
+    foreach my $param (qw/check node_element_cb hash_element_cb 
                           list_element_cb check_list_element_cb node_content_cb
+                          node_content_hook list_element_hook hash_element_hook
                           experience auto_vivify up_cb/, @value_cb) {
         $self->{$param} = $args{$param} if defined $args{$param};
 	delete $args{$param} ; # may exists but be undefined
@@ -380,6 +447,15 @@ sub new {
           unless defined $self->{$param} ;
     }
 
+    # this parameter is optional and does not need a fallback
+    $self->{node_dispatch_cb} = delete $args{node_dispatch_cb} || {} ;
+    
+    croak __PACKAGE__,"->new: node_dispatch_cb is not a hash ref" 
+	unless ref($self->{node_dispatch_cb}) eq 'HASH';
+
+    croak __PACKAGE__,"->new: unexpected check: $self->{check}" 
+	unless $self->{check} =~ /yes|no|skip/;
+
     croak __PACKAGE__,"->new: unexpected parameter: ",join (' ',keys %args)
       if scalar %args ;
 
@@ -391,6 +467,8 @@ sub create_fallback {
     my $self = shift ;
     my $fallback = shift;
 
+    map { $self->{$_} = sub {} } qw/node_content_hook hash_element_hook list_element_hook/;
+ 
     return if not defined $fallback or $fallback eq 'none' ;
 
     my $done = 0 ;
@@ -440,7 +518,11 @@ sub create_fallback {
 
 =head2 scan_node ($data_r,$node)
 
-Explore the node and call C<node_element_cb> passing all element names.
+Explore the node and call either C<node_dispatch_cb> (if the node class 
+name matches the dispatch_node hash) B<or> (e.g. xor) C<node_element_cb> passing 
+all element names.
+
+After the first callback has returned, C<up_cb> will be called.
 
 =cut
 
@@ -462,11 +544,21 @@ sub scan_node {
 	return unless defined $node ;
     }
 
-    my @element_list= $node->get_element_name(for => $self->{experience}) ;
+    my $config_class = $node->config_class_name ;
+    my $node_dispatch_cb = $self->{node_dispatch_cb}{$config_class} ;
+    
+    my $actual_cb = $node_dispatch_cb || $self->{node_content_cb};
+    
+    my @element_list= $node->get_element_name(
+        for => $self->{experience}, 
+        check => $self->{check}
+    ) ;
+
+    $self->{node_content_hook}->($self, $data_r,$node,@element_list) ;
 
     # we could add here a "last element" call-back, but it's not
     # very useful if the last element is a hash.
-    $self->{node_content_cb}->($self, $data_r,$node,@element_list) ;
+    $actual_cb->($self, $data_r,$node,@element_list) ;
 
     $self->{up_cb}->($self, $data_r,$node) ;
 }
@@ -485,7 +577,6 @@ sub scan_element {
 
     my $element_type = $node->element_type($element_name);
 
-    return unless defined $element_type; # element may not be initialized
     my $autov = $self->{auto_vivify} ;
 
     #print "scan_element $element_name ";
@@ -493,23 +584,26 @@ sub scan_element {
         #print "type hash\n";
         my @keys = $self->get_keys($node,$element_name) ;
         # if hash element grab keys and perform callback
+        $self->{hash_element_hook}->($self, $data_r,$node,$element_name,@keys);
         $self->{hash_element_cb}->($self, $data_r,$node,$element_name,@keys);
     }
     elsif ($element_type eq 'list') {
         #print "type list\n";
         my @keys = $self->get_keys($node,$element_name) ;
+        $self->{list_element_hook}->($self, $data_r,$node,$element_name, @keys);
         $self->{list_element_cb}->($self, $data_r,$node,$element_name, @keys);
     }
     elsif ($element_type eq 'check_list') {
         #print "type list\n";
-	my $cl_elt = $node->fetch_element($element_name) ;
+	my $cl_elt = $node->fetch_element(name => $element_name, check => $self->{check}) ;
         $self->{check_list_element_cb}->($self, $data_r,$node,$element_name, undef, $cl_elt);
     }
     elsif ($element_type eq 'node') {
         #print "type object\n";
 	# avoid auto-vivification
 	my $next_obj = ($autov or $node->is_element_defined($element_name))
-	             ? $node->fetch_element($element_name) : undef ;
+	             ? $node->fetch_element(name => $element_name, check => $self->{check}) 
+	             : undef ;
 
         # if obj element, cb
         $self->{node_element_cb}-> ($self, $data_r, $node,
@@ -518,12 +612,13 @@ sub scan_element {
     elsif ($element_type eq 'warped_node') {
         #print "type warped\n";
 	my $next_obj = ($autov or $node->is_element_defined($element_name))
-	             ? $node->fetch_element($element_name) : undef ;
+	             ? $node->fetch_element(name => $element_name, check => $self->{check}) 
+	             : undef ;
         $self->{node_element_cb}-> ($self, $data_r,$node,
 				    $element_name, undef, $next_obj) ;
     }
     elsif ($element_type eq 'leaf') {
-	my $next_obj = $node->fetch_element($element_name) ;
+	my $next_obj = $node->fetch_element(name => $element_name, check => $self->{check});
 	my $type = $next_obj->value_type ;
 	return unless $type;
 	my $cb_name = $type.'_value_cb' ;
@@ -550,11 +645,11 @@ sub scan_hash {
     assert_like($node->element_type($element_name), qr/(hash|list)/);
 
     #print "scan_hash ",$node->name," element $element_name key $key ";
-    my $item = $node -> fetch_element($element_name) ;
+    my $item = $node -> fetch_element(name => $element_name, check => $self->{check}) ;
 
 
     my $cargo_type = $item->cargo_type($element_name);
-    my $next_obj = $item->fetch_with_id($key) ;
+    my $next_obj = $item->fetch_with_id(index  => $key, check => $self->{check}) ;
 
     if ($cargo_type =~ /node$/) {
         #print "type object or warped\n";
@@ -587,7 +682,7 @@ sub scan_list {
 =head2 get_keys ($node, $element_name)
 
 Returns an list containing the sorted keys of a hash element or returns
-an list containning (0.. last_index) of an list element.
+an list containing (0.. last_index) of an list element.
 
 Throws an exception if element is not an list or a hash element.
 
@@ -597,9 +692,9 @@ sub get_keys {
     my ($self,$node,$element_name) = @_ ;
 
     my $element_type = $node->element_type($element_name);
-    my $item = $node->fetch_element($element_name) ;
+    my $item = $node->fetch_element(name => $element_name, check => $self->{check}) ;
 
-    return $item->get_all_indexes 
+    return $item->fetch_all_indexes 
       if    $element_type eq 'hash' 
 	 || $element_type eq 'list' ;
 
@@ -654,4 +749,4 @@ L<Config::Model::ListId>,
 L<Config::Model::CheckList>,
 L<Config::Model::Value>
 
-=cut
\ No newline at end of file
+=cut
