@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 
-#    Copyright (c) 2006-2008 Dominique Dumont.
+#    Copyright (c) 2006-2012 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -29,7 +29,7 @@
 
 package Config::Model::TermUI ;
 {
-  $Config::Model::TermUI::VERSION = '2.005';
+  $Config::Model::TermUI::VERSION = '2.021';
 }
 
 use Carp;
@@ -40,6 +40,7 @@ use Term::ReadLine;
 
 use base qw/Config::Model::SimpleUI/ ;
 
+=encoding utf8
 
 =head1 NAME
 
@@ -47,7 +48,7 @@ Config::Model::TermUI - Provides Config::Model UI  la Term::ReadLine
 
 =head1 VERSION
 
-version 2.005
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -225,7 +226,7 @@ my $cd_completion_sub = sub {
 	}
     }
     elsif ($new_type eq 'hash' or $new_type eq 'list') {
-	my @idx = $new_item -> get_all_indexes ;
+	my @idx = $new_item -> fetch_all_indexes ;
 	if (@idx) {
 	    my $quote = $line =~ /"$/ ? '' : '"';
 	    my @tmp = map { /\s/ ? qq($quote$_" ) : qq($_ ); } @idx ;
