@@ -9,7 +9,7 @@
 #
 package Config::Model::Tester;
 {
-  $Config::Model::Tester::VERSION = '2.011';
+  $Config::Model::Tester::VERSION = '2.021';
 }
 
 use Test::More;
@@ -48,7 +48,7 @@ sub setup_test {
     rmtree($wr_root);
     mkpath( $wr_root, { mode => 0755 } );
 
-    my $wr_dir    = $wr_root . '/test-' . $t_name;
+    my $wr_dir    = $wr_root . '/test-' . $t_name.'/';
     my $conf_file ;
     $conf_file = "$wr_dir/$conf_dir/$conf_file_name" if defined $conf_file_name;
 
@@ -191,7 +191,7 @@ sub run_model_test {
         {
             local $Config::Model::Value::nowarning = 1;
             &$risky;
-            ok( 1, "Ran dump_tree (no warning check" );
+            ok( 1, "Ran dump_tree (no warning check)" );
         }
         else {
             warnings_like { &$risky; } $t->{dump_warnings}, "Ran dump_tree";
@@ -222,7 +222,7 @@ sub run_model_test {
             } 
         }
 
-        $inst->write_back( );
+        $inst->write_back( force => 1 );
         ok( 1, "$model_test write back done" );
         
         if (my $fc = $t->{file_content}) {
@@ -231,6 +231,18 @@ sub run_model_test {
             } 
         }
 
+        if (my $fc = $t->{file_contents_like}) {
+            foreach my $f (keys %$fc) {
+                file_contents_like $wr_dir.$f,  $fc->{$f},  "check that $f matches regexp";
+            } 
+        }
+
+        if (my $fc = $t->{file_contents_unlike}) {
+            foreach my $f (keys %$fc) {
+                file_contents_unlike $wr_dir.$f,  $fc->{$f},  "check that $f does not match regexp";
+            } 
+        }
+
         my @new_file_list;
         if ( -d $ex_data ) {
 
@@ -240,12 +252,13 @@ sub run_model_test {
             my @new_file_list = list_test_files($debian_dir) ;
             $t->{file_check_sub}->( \@file_list )
               if defined $t->{file_check_sub};
-            eq_or_diff( \@new_file_list, \@file_list,
+            eq_or_diff( \@new_file_list, [ sort @file_list ],
                 "check added or removed files" );
         }
 
         # create another instance to read the conf file that was just written
-        my $wr_dir2 = $wr_dir . '-w';
+        my $wr_dir2 = $wr_dir ;
+        $wr_dir2 =~ s!/$!-w/!;
         dircopy( $wr_dir, $wr_dir2 )
           or die "can't copy from $wr_dir to $wr_dir2: $!";
 
@@ -298,7 +311,7 @@ sub run_tests {
     $log  = 1 if $arg =~ /l/;
     $show = 1 if $arg =~ /s/;
 
-    my $log4perl_user_conf_file = $ENV{HOME} . '/.log4config-model';
+    my $log4perl_user_conf_file = ($ENV{HOME} || '') . '/.log4config-model';
 
     if ( $log and -e $log4perl_user_conf_file ) {
         Log::Log4perl::init($log4perl_user_conf_file);
@@ -337,7 +350,7 @@ Config::Model::Tester - Test framework for Config::Model
 
 =head1 VERSION
 
-version 2.011
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -358,7 +371,7 @@ version 2.011
 =head1 DESCRIPTION
 
 This class provides a way to test configuration models with tests files. 
-This class was designed to tests several models and severals tests 
+This class was designed to tests several models and several tests 
 cases per model.
 
 A specific layout for test files must be followed
@@ -459,6 +472,8 @@ Check for config data warning. You should pass the list of expected warnings.
 E.g.  
 
     load_warnings => [ qr/Missing/, (qr/deprecated/) x 3 , ],
+    
+Use an empty array_ref to masks load warnings.
 
 =item *
 
@@ -486,7 +501,7 @@ data. Use C<dump_errors> if you expect issues:
 
 =item *
 
-Likewise, specify any expected warnings:
+Likewise, specify any expected warnings (note the list must contain only C<qr> stuff):
 
         dump_warnings => [ (qr/deprecated/) x 3 ],
 
@@ -529,6 +544,9 @@ Verify annotation extracted from the configuration file comments:
 =item *
 
 Write back the config data in C<< wr_root/<subtest name>/ >>. 
+Note that write back is forced, so the tested configuration files are
+written back even if the configuration values were not changed during the test.
+
 You can skip warning when writing back with:
 
     no_warnings => 1,
@@ -540,6 +558,14 @@ Check the content of the written files(s) with L<Test::File::Contents>:
    file_content => { 
             "/home/foo/my_arm.conf" => "really big string" ,
         }
+   
+   file_contents_like => {
+            "/home/foo/my_arm.conf" => qr/should be there/ ,
+   }
+
+   file_contents_unlike => {
+            "/home/foo/my_arm.conf" => qr/should NOT be there/ ,
+   }
 
 =item *
 
