@@ -9,11 +9,14 @@
 #
 package Config::Model::Value ;
 {
-  $Config::Model::Value::VERSION = '2.011';
+  $Config::Model::Value::VERSION = '2.021';
 }
 
+use 5.10.1 ;
+
 use Any::Moose;
 use Any::Moose '::Util::TypeConstraints' ;
+use Any::Moose 'X::StrictConstructor' ;
 use namespace::autoclean;
 
 use Data::Dumper ();
@@ -30,6 +33,7 @@ extends qw/Config::Model::AnyThing/ ;
 
 my $logger = get_logger("Tree::Element::Value") ;
 my $change_logger = get_logger("Anything::Change") ;
+my $fix_logger = get_logger("Anything::Fix") ;
 
 our $nowarning = 0; # global variable to silence warnings. Only used for tests
 
@@ -119,18 +123,19 @@ sub notify_change {
     my %args = @_ ;
     my $check_done = $args{check_done} || 0 ;
 
+    return if $self->instance->initial_load and not $args{really};
     
     $change_logger->debug("called while needs_check is ",$self->needs_check,
 	" for ",$self->name) 
 	if $change_logger->is_debug ;
     $self->needs_check(1) unless $check_done;
-    $self->SUPER::notify_change(%args) ;
+    $self->SUPER::notify_change(%args, value_type => $self->value_type) ;
 
     # notify all warped or computed objects that depends on me 
     foreach my $s ($self->get_depend_slave) {
 	$change_logger->debug("calling notify_change on slave ",$s->name) 
 	    if $change_logger->is_debug ;
-	$s -> notify_change ;
+	$s -> notify_change(note => 'master triggered changed') ;
     }  ;
 }
 
@@ -291,14 +296,18 @@ sub set_migrate_from {
 sub migrate_value {
     my $self = shift ;
 
-    my $i = $self->instance;
+    return if $self->{migration_done} ;
+    return if $self->instance->initial_load ;
+    $self->{migration_done} =1 ;
 
     # avoid warning when reading deprecated values
     my $result = $self->{_migrate_from} -> compute (check => 'no');
 
+    return undef unless defined $result;
+
     # check if the migrated result fits with the constraints of the
     # Value object
-    my $ok = $self->check_value(value => $result, mode => 'allow_undef') ;
+    my $ok = $self->check_value(value => $result) ;
 
     #print "check result: $ok\n";
     if (not $ok) {
@@ -312,6 +321,10 @@ sub migrate_value {
 		     );
     }
 
+    # old value is always undef when this method is called
+    $self->notify_change(note =>'migrated value', new => $result );
+    $self->{data} = $result ;
+
     return $ok ? $result : undef ;
 }
 
@@ -1007,11 +1020,11 @@ sub apply_fixes {
     my $self = shift ; 
 
     if ($logger->is_debug) {
-        $logger->debug( "called for ".$self->location ) ;
+        $fix_logger->debug( "called for ".$self->location ) ;
     }
 
     $self->check_value(value => $self->{data}, fix => 1);
-    $self->notify_change ;
+    # $self->notify_change ;
 }
 
 
@@ -1019,9 +1032,14 @@ sub apply_fixes {
 sub apply_fix {
     my ( $self, $fix, $value ) = @_;
 
-    local $_ = $value;
+    local $_ = $value; # used inside $fix sub ref
 
-    $logger->info( $self->location . ": Applying fix '$fix'" );
+    if ($fix_logger->is_debug){ 
+	my $str = $fix ;
+	$str =~ s/\n/ /g ;
+	$fix_logger->info( $self->location . ": Applying fix '$str'" );
+    }
+    
     eval($fix);
     if ($@) {
         Config::Model::Exception::Model->throw(
@@ -1031,7 +1049,17 @@ sub apply_fix {
     }
 
     $self->{data}  = $_ ;
-    $self->notify_change ;
+
+    if ($fix_logger->is_trace){ 
+	$fix_logger->trace( "fix change: '" . ($value // '<undef>') ."' -> '"
+	    . ($self->{data} // '<undef>'). "'" );
+    }
+    
+    $self->notify_change(
+        old => $value // $self->_fetch_std, 
+        new => $self->{data} // $self->_fetch_std,
+        note => 'applied fix'
+    ) ;
     # $self->store(value => $_, check => 'no');  # will update $self->{fixes}
 }
 
@@ -1070,10 +1098,21 @@ sub check {
     }
 
     my $warn = $self->{warning_list};
-    map {
-	my $str = defined $value ? "'$value'" : '<undef>';
-	warn "Warning in '" . $self->location . "' value $str: $_\n";
-    } @$warn if @$warn and not $nowarning and not $silent;
+    
+    # old_warn is used to avoid warning the user several times for the 
+    # same reason. We take care to clean up this hash each time this routine
+    # is run
+    my $old_warn = $self->{old_warning_hash} || {} ;
+    my %warn_h ;
+    if (@$warn and not $nowarning and not $silent) {
+        foreach my $w (@$warn) {
+            $warn_h{$w} = 1 ;
+            next if $old_warn->{$w} ;
+            my $str = defined $value ? "'$value'" : '<undef>';
+            warn "Warning in '" . $self->location . "' value $str: $w\n";
+        }
+    }
+    $self->{old_warning_hash} = \%warn_h ;
 
     my $e = $self->{error_list} ;
     return wantarray ? @$e : not scalar @$e ;
@@ -1117,12 +1156,17 @@ sub store {
 	    $self->{layered} = $value ;
 	} 
 	elsif ($self->instance->preset) {
-	    $self->notify_change(check_done => 1) if $notify_change;
+	    $self->notify_change(check_done => 1,old => $self->{data}, new => $value)
+                if $notify_change;
 	    $self->{preset} = $value ;
 	} 
 	else {
 	    no warnings 'uninitialized';
-	    $self->notify_change(check_done => 1) if $notify_change;
+	    $self->notify_change(
+                check_done => 1,
+                old => $self->{data} // $self->_fetch_std, 
+                new => $value // $self->_fetch_std
+            ) if $notify_change;
 	    $self->{data} = $value ; # may be undef
 	}
 	
@@ -1275,7 +1319,7 @@ sub fetch_custom {
 
 sub fetch_standard {
     my $self = shift ;
-    my $pre_fetch = $self->_pre_fetch ;
+    my $pre_fetch = $self->_fetch_std ;
     my $v = defined $pre_fetch                 ? $pre_fetch 
           : defined $self->{layered}           ? $self->{layered}
           : $self->compute_is_upstream_default ? $self->perform_compute
@@ -1299,7 +1343,7 @@ sub _init {
 
 # returns something that needs to be written to config file
 # unless overridden by user data
-sub _pre_fetch {
+sub _fetch_std {
     my ($self, $mode, $check) = @_ ;
 
     #$self->_init ;
@@ -1348,16 +1392,22 @@ sub _fetch {
     $logger->debug("called for ".$self->location) if $logger->is_debug ;
 
     # always call to perform submit_to_warp
-    my $pref = $self->_pre_fetch($mode, $check) ;
+    my $pref = $self->_fetch_std($mode, $check) ;
+
+    my $data = $self->{data} ;
+    my $old_data = $data // $self->{_std_backup} ;
+    if (defined $pref  and not defined $old_data) {
+        $self->{_std_backup} = $pref ;
+        $self->notify_change(old => $old_data, new => $pref, note => "use standard value") ;
+    }
+
     my $known_upstream = defined $self->{layered} ? $self->{layered}
 		       : $self->compute_is_upstream_default ? $self->perform_compute 
                        :                            $self->{upstream_default} ;
     my $std = defined $pref ? $pref : $known_upstream ;
-    my $data = $self->{data} ;
 
     if (not defined $data and defined $self->{_migrate_from}) {
 	$data =  $self->migrate_value ;
-	$self->notify_change ;
     }
 
     foreach my $k (keys %old_mode) {
@@ -1610,7 +1660,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.011
+version 2.021
 
 =head1 SYNOPSIS
 
@@ -1849,9 +1899,9 @@ String. Issue a warning to user with the specified string any time a value is se
 
 =item warn_unless
 
-A bit like C<warn_if_match>. The hash key is not a regexp but a label to help users.
-The hash ref containd some Perl code that is evaluated to perform the test. A warning will be issued if 
-the code returns false. 
+A bit like C<warn_if_match>. The hash key is not a regexp but a label to
+help users. The hash ref contains some Perl code that is evaluated to
+perform the test. A warning will be issued if the code returns false.
 
 C<$_> will contains the value to check. C<$self> will contain the C<Config::Model::Value> object.
 
