@@ -70,17 +70,11 @@ To upgrade your file from an old spec, run:
       },
       'Upstream-Contact',
       {
-        'migrate_keys_from' => '- Upstream-Maintainer',
         'cargo' => {
           'value_type' => 'uniline',
-          'migrate_from' => {
-            'formula' => '$maintainer',
-            'variables' => {
-              'maintainer' => '- Upstream-Maintainer:&index'
-            }
-          },
           'type' => 'leaf'
         },
+        'migrate_values_from' => '- Upstream-Maintainer',
         'type' => 'list',
         'description' => 'The preferred address(es) to reach the upstream project. May be free-form text, but by convention will usually be written as a list of RFC5822 addresses or URIs.'
       },
@@ -136,6 +130,7 @@ To upgrade your file from an old spec, run:
           'config_class_name' => 'Debian::Dpkg::Copyright::Content'
         },
         'ordered' => '1',
+        'warn_if_key_match' => '[\\[\\]\\|]',
         'type' => 'hash',
         'description' => 'Patterns indicating files having the same license and sharing copyright holders. See "File patterns" below',
         'index_type' => 'string'
@@ -169,18 +164,12 @@ To upgrade your file from an old spec, run:
       },
       'Upstream-Maintainer',
       {
-        'migrate_keys_from' => '- Maintainer',
         'cargo' => {
           'value_type' => 'uniline',
-          'migrate_from' => {
-            'formula' => '$maintainer',
-            'variables' => {
-              'maintainer' => '- Maintainer:&index'
-            }
-          },
           'type' => 'leaf'
         },
         'status' => 'deprecated',
+        'migrate_values_from' => '- Maintainer',
         'type' => 'list'
       },
       'Upstream-Source',
@@ -201,7 +190,7 @@ To upgrade your file from an old spec, run:
           'type' => 'node',
           'config_class_name' => 'Debian::Dpkg::Copyright::LicenseSpec'
         },
-        'allow_keys_matching' => '^[\\w\\-\\.+]+$',
+        'warn_if_key_match' => '\\s',
         'type' => 'hash',
         'index_type' => 'string'
       },
