@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -14,7 +14,8 @@
       '.*',
       {
         'value_type' => 'string',
-        'type' => 'leaf'
+        'type' => 'leaf',
+        'description' => 'license short_name. Example: GPL-1 LPL-2.1+'
       }
     ],
     'name' => 'Debian::Dpkg::Copyright::LicenseSpec',
@@ -29,6 +30,21 @@
     'element' => [
       'text',
       {
+        'compute' => {
+          'undef_is' => '\'\'',
+          'use_eval' => '1',
+          'formula' => 'require Software::License ;
+my $h = { 
+  short_name => &index( - ), 
+  holder => \'foo\' 
+} ;
+
+# no need to fail if short_name is unknown
+eval {
+  Software::License->new($h)->summary ; 
+} ;',
+          'allow_override' => '1'
+        },
         'value_type' => 'string',
         'type' => 'leaf',
         'description' => 'Full license text.'
