@@ -1,13 +1,13 @@
-# 
-# This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
-# This is free software, licensed under:
-# 
-#   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
 # -*- cperl -*-
+#
+# This file is part of Config-Model
+#
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
+#
+# This is free software, licensed under:
+#
+#   The GNU Lesser General Public License, Version 2.1, February 1999
+#
 
 # this file is used by test script
 
@@ -17,7 +17,8 @@
    element => [
 	       [qw/aa2 ab2 ac2 ad2 Z/] =>
 	       { type => 'leaf', value_type => 'string' }
-	      ]
+	      ],
+    description => [ Z => 'Z comme zorro' ],
   ],
 
   [
@@ -99,7 +100,11 @@
   [
    name => 'Master',
    permission => [ [qw/tree_macro warp/] => 'advanced'] ,
-   class_description => "Master description",
+   class_description => "Master configuration class is a wonderful test class\n"
+     . "widely used in Config::Model self tests",
+    copyright => [ "2005-2011, Dominique Dumont" ],
+    license => 'LGPL-2',
+    author => 'Dominique Dumont' ,
    level      => [ [qw/lista hash_a tree_macro int_v/] => 'important' ],
    element => [
 	       std_id => { type => 'hash',
@@ -189,7 +194,8 @@
 			       }
 	      ],
    description => [
-		   tree_macro => 'controls behavior of other elements'
+		   tree_macro => 'controls behavior of other elements',
+		   hidden_string => 'shy text',
 		  ]
    ],
 ] ;
